assert(LibStub, "LibDraggable-1.0 requires LibStub")

local MAJOR, MINOR = "LibDraggable-1.0", 1
local LibDraggable = LibStub:NewLibrary(MAJOR, MINOR)
if not LibDraggable then return end

LibDraggable.parents = LibDraggable.parents or {}
LibDraggable.savedTabs = LibDraggable.savedTabs or {}
LibDraggable.buttons = LibDraggable.buttons or {}

local select = select

local function getFrame(frame)
	return LibDraggable.parents[frame] or frame
end

local function savePosition(frame)
	local obj, tab = nil, {}
	tab.point, obj, tab.relativePoint, tab.x, tab.y = frame:GetPoint()
	tab.relativeTo = obj and obj:GetName() or "UIParent"
	LibDraggable.savedTabs[frame] = tab
end

local function loadPosition(frame)
	local tab = LibDraggable.savedTabs[frame]
	if tab then
		local point, obj, relativePoint, x, y = tab.point, _G[tab.relativeTo], tab.relativePoint, tab.x, tab.y
		if point and obj and relativePoint and x and y then
			frame:ClearAllPoints()
			frame:SetPoint(point, obj, relativePoint, x, y)
		end
	end
end

local function mouseDown(frame)
	frame = getFrame(frame)
	local savedButton = LibDraggable.buttons[frame]
	if (savedButton and button ~= "LeftButton") or (savedButton and button ~= savedButton) then
		return
	end
	if frame:IsMovable() then
		frame:StartMoving()
	end
end

local function mouseUp(frame, button)
	frame = getFrame(frame)
	local savedButton = LibDraggable.buttons[frame]
	if (savedButton and button ~= "LeftButton") or (savedButton and button ~= savedButton) then
		return
	end
	frame:StopMovingOrSizing()
	if LibDraggable.savedTabs[frame] then
		savePosition(frame)
	end
end

function LibDraggable:EnableIndirectDragging(frame, ...)
	frame:SetMovable(true)
	local child
	for i = 1, select("#", ...) do
		child = select(i, ...)
		if child then
			child:SetMovable(true)
			child:SetClampedToScreen(false)
			child:EnableMouse(true)
			child:HookScript("OnMouseDown", mouseDown)
			child:HookScript("OnMouseUp", mouseUp)
			self.parents[child] = frame
		end
	end
end

function LibDraggable:EnableDragging(frame, ...)
	self:EnableIndirectDragging(frame, ...)
	frame:EnableMouse(true)
	frame:HookScript("OnMouseDown", mouseDown)
	frame:HookScript("OnMouseUp", mouseUp)
end

function LibDraggable:DisableDragging(frame, ...)
	frame:SetMovable(false)
	local child
	for i = 1, select("#", ...) do
		child = select(i, ...)
		if child then
			child:SetMovable(false)
		end
	end
end

function LibDraggable:EnableSavePosition(frame)
	frame = getFrame(frame)
	frame:HookScript("OnShow", loadPosition)
	self.savedTabs[frame] = {}
end

function LibDraggable:DisableSavePosition(frame)
	self.savedTabs[getFrame(frame)] = nil
end

function LibDraggable:SetClampedToScreen(frame, clamped)
	getFrame(frame):SetClampedToScreen(clamped)
end

function LibDraggable:SetDragButton(frame, button)
	frame = getFrame(frame)
	self.buttons[frame] = button
end